; DO NOT EDIT DIRECTLY.
; This file was automatically generated.
; This file is for the Microsoft linker.
; Sat Feb 25 19:55:47 2017
EXPORTS
   marpa_check_version
   marpa_version
   marpa_c_init
   marpa_c_error
   marpa_g_new
   marpa_g_force_valued
   marpa_g_ref
   marpa_g_unref
   marpa_g_start_symbol
   marpa_g_start_symbol_set
   marpa_g_highest_symbol_id
   marpa_g_symbol_is_accessible
   marpa_g_symbol_is_nullable
   marpa_g_symbol_is_nulling
   marpa_g_symbol_is_productive
   marpa_g_symbol_is_start
   marpa_g_symbol_is_terminal_set
   marpa_g_symbol_is_terminal
   marpa_g_symbol_new
   marpa_g_highest_rule_id
   marpa_g_rule_is_accessible
   marpa_g_rule_is_nullable
   marpa_g_rule_is_nulling
   marpa_g_rule_is_loop
   marpa_g_rule_is_productive
   marpa_g_rule_length
   marpa_g_rule_lhs
   marpa_g_rule_new
   marpa_g_rule_rhs
   marpa_g_rule_is_proper_separation
   marpa_g_sequence_min
   marpa_g_sequence_new
   marpa_g_sequence_separator
   marpa_g_symbol_is_counted
   marpa_g_rule_rank_set
   marpa_g_rule_rank
   marpa_g_rule_null_high_set
   marpa_g_rule_null_high
   marpa_g_completion_symbol_activate
   marpa_g_nulled_symbol_activate
   marpa_g_prediction_symbol_activate
   marpa_g_symbol_is_completion_event
   marpa_g_symbol_is_completion_event_set
   marpa_g_symbol_is_nulled_event
   marpa_g_symbol_is_nulled_event_set
   marpa_g_symbol_is_prediction_event
   marpa_g_symbol_is_prediction_event_set
   marpa_g_precompute
   marpa_g_is_precomputed
   marpa_g_has_cycle
   marpa_r_new
   marpa_r_ref
   marpa_r_unref
   marpa_r_start_input
   marpa_r_alternative
   marpa_r_earleme_complete
   marpa_r_current_earleme
   marpa_r_earleme
   marpa_r_earley_set_value
   marpa_r_earley_set_values
   marpa_r_furthest_earleme
   marpa_r_latest_earley_set
   marpa_r_latest_earley_set_value_set
   marpa_r_latest_earley_set_values_set
   marpa_r_completion_symbol_activate
   marpa_r_earley_item_warning_threshold_set
   marpa_r_earley_item_warning_threshold
   marpa_r_expected_symbol_event_set
   marpa_r_is_exhausted
   marpa_r_nulled_symbol_activate
   marpa_r_prediction_symbol_activate
   marpa_r_terminals_expected
   marpa_r_terminal_is_expected
   marpa_r_progress_report_reset
   marpa_r_progress_report_start
   marpa_r_progress_report_finish
   marpa_r_progress_item
   marpa_b_new
   marpa_b_ref
   marpa_b_unref
   marpa_b_ambiguity_metric
   marpa_b_is_null
   marpa_o_new
   marpa_o_ref
   marpa_o_unref
   marpa_o_ambiguity_metric
   marpa_o_is_null
   marpa_o_high_rank_only_set
   marpa_o_high_rank_only
   marpa_o_rank
   marpa_t_new
   marpa_t_ref
   marpa_t_unref
   marpa_t_next
   marpa_t_parse_count
   marpa_v_new
   marpa_v_ref
   marpa_v_unref
   marpa_v_step
   marpa_g_event
   marpa_g_event_count
   marpa_g_error
   marpa_g_error_clear
   marpa_g_default_rank_set
   marpa_g_default_rank
   marpa_g_symbol_rank_set
   marpa_g_symbol_rank
   marpa_g_zwa_new
   marpa_g_zwa_place
   marpa_r_zwa_default
   marpa_r_zwa_default_set
   marpa_g_highest_zwa_id
   marpa_r_clean
   marpa_g_symbol_is_valued_set
   marpa_g_symbol_is_valued
   marpa_v_symbol_is_valued_set
   marpa_v_symbol_is_valued
   marpa_v_rule_is_valued_set
   marpa_v_rule_is_valued
   marpa_v_valued_force
   _marpa_g_nsy_is_start
   _marpa_g_nsy_is_nulling
   _marpa_g_nsy_is_lhs
   _marpa_g_xsy_nulling_nsy
   _marpa_g_xsy_nsy
   _marpa_g_nsy_is_semantic
   _marpa_g_source_xsy
   _marpa_g_nsy_lhs_xrl
   _marpa_g_nsy_xrl_offset
   _marpa_g_rule_is_keep_separation
   _marpa_g_nsy_count
   _marpa_g_irl_count
   _marpa_g_irl_lhs
   _marpa_g_irl_length
   _marpa_g_irl_rhs
   _marpa_g_rule_is_used
   _marpa_g_irl_is_virtual_lhs
   _marpa_g_irl_is_virtual_rhs
   _marpa_g_virtual_start
   _marpa_g_virtual_end
   _marpa_g_source_xrl
   _marpa_g_real_symbol_count
   _marpa_g_irl_semantic_equivalent
   _marpa_g_irl_rank
   _marpa_g_nsy_rank
   _marpa_g_ahm_count
   _marpa_g_ahm_irl
   _marpa_g_ahm_position
   _marpa_g_ahm_postdot
   _marpa_r_is_use_leo
   _marpa_r_is_use_leo_set
   _marpa_r_trace_earley_set
   _marpa_r_earley_set_size
   _marpa_r_earley_set_trace
   _marpa_r_earley_item_trace
   _marpa_r_earley_item_origin
   _marpa_r_leo_predecessor_symbol
   _marpa_r_leo_base_origin
   _marpa_r_leo_base_state
   _marpa_r_postdot_symbol_trace
   _marpa_r_first_postdot_item_trace
   _marpa_r_next_postdot_item_trace
   _marpa_r_postdot_item_symbol
   _marpa_r_first_token_link_trace
   _marpa_r_next_token_link_trace
   _marpa_r_first_completion_link_trace
   _marpa_r_next_completion_link_trace
   _marpa_r_first_leo_link_trace
   _marpa_r_next_leo_link_trace
   _marpa_r_source_predecessor_state
   _marpa_r_source_token
   _marpa_r_source_leo_transition_symbol
   _marpa_r_source_middle
   _marpa_b_and_node_count
   _marpa_b_and_node_middle
   _marpa_b_and_node_parent
   _marpa_b_and_node_predecessor
   _marpa_b_and_node_cause
   _marpa_b_and_node_symbol
   _marpa_b_and_node_token
   _marpa_b_top_or_node
   _marpa_b_or_node_set
   _marpa_b_or_node_origin
   _marpa_b_or_node_irl
   _marpa_b_or_node_position
   _marpa_b_or_node_is_whole
   _marpa_b_or_node_is_semantic
   _marpa_b_or_node_first_and
   _marpa_b_or_node_last_and
   _marpa_b_or_node_and_count
   _marpa_o_and_order_get
   _marpa_o_or_node_and_node_count
   _marpa_o_or_node_and_node_id_by_ix
   _marpa_t_size
   _marpa_t_nook_or_node
   _marpa_t_nook_choice
   _marpa_t_nook_parent
   _marpa_t_nook_cause_is_ready
   _marpa_t_nook_predecessor_is_ready
   _marpa_t_nook_is_cause
   _marpa_t_nook_is_predecessor
   _marpa_v_trace
   _marpa_v_nook
   _marpa_tag
   marpa_debug_level_set
   marpa_debug_handler_set
